#include "triangle_mesh_type.h"
#include <vcg/math/random_generator.h>
#include <vcg/complex/algorithms/create/platonic.h>
#include <vcg/complex/algorithms/refine.h>

using namespace vcg;
math::MarsenneTwisterRNG rnd;
void GenerateBox(MyTriMesh &mesh, float size)
{
  mesh.Clear();
  tri::Box<MyTriMesh>(mesh, Box3f(Point3f(0,0,0), Point3f(size,size,size)));
  tri::UpdateTopology<MyTriMesh>::FaceFace(mesh);
  
  for(int i=0;i<7;++i)
    tri::Refine<MyTriMesh, tri::MidPoint<MyTriMesh> >(mesh,tri::MidPoint<MyTriMesh>(&mesh),0.0f);
			
  tri::UpdateNormal<MyTriMesh>::PerVertexNormalizedPerFace(mesh);
  tri::UpdateBounding<MyTriMesh>::Box(mesh);
}


void RandomizeMesh(MyTriMesh &mesh, float threshold)
{
  for (int i = 0; i < mesh.vert.size(); i++)
  {
    Point3f randomVector = math::GeneratePointInUnitBallUniform<float>(rnd);
    mesh.vert[i].P() += randomVector*threshold; 
  }
  tri::UpdateNormal<MyTriMesh>::PerVertexNormalizedPerFace(mesh);
  
}

void SearchFittingPlane(MyTriMesh &mesh, Plane3f &fittingPlane, float &inlierFraction)
{
  // First I take three random points over my mesh
  // checking that they are far apart and not aligned
  // Then I compute the plane passing through them
  // Then I check how many points are close to the plane
  // and I keep the plane with the most points
  // I repeat this process for a number of times

  int p0,p1,p2;
  float distanceThreshold = mesh.bbox.Diag()*0.1;

  p0 = rnd.generate(mesh.vert.size());
  while(true)
  {
    p1 = rnd.generate(mesh.vert.size());
    if (Distance(mesh.vert[p0].P(),mesh.vert[p1].P()) > distanceThreshold)  
      break;
  }
 while(true)
  {
    p2 = rnd.generate(mesh.vert.size());
    
    if ( (Distance(mesh.vert[p0].P(),mesh.vert[p2].P()) > distanceThreshold) &&
    (Distance(mesh.vert[p1].P(),mesh.vert[p2].P()) > distanceThreshold) )
      break;
  }

  fittingPlane.Init(mesh.vert[p0].P(),mesh.vert[p1].P(),mesh.vert[p2].P());
  float threshold = mesh.bbox.Diag()*0.02;

  int checkingPoints = 1000; // number of points to check
  int pointsOnPlane = 0;
    for(int i=0;i<checkingPoints;++i)
  {
    int p = rnd.generate(mesh.vert.size());
    if (fabs(SignedDistancePlanePoint(fittingPlane,mesh.vert[p].P())) < threshold)
        pointsOnPlane++;
  }
  inlierFraction = (float)pointsOnPlane/(float)checkingPoints; 
}